package de.fitko.fitconnect.api.services;

import com.nimbusds.jose.jwk.RSAKey;
import de.fitko.fitconnect.api.domain.auth.OAuthToken;
import de.fitko.fitconnect.api.domain.model.metadata.Metadata;
import de.fitko.fitconnect.api.domain.model.metadata.attachment.Attachment;
import de.fitko.fitconnect.api.domain.model.metadata.data.Data;
import de.fitko.fitconnect.api.domain.model.submission.Submission;
import de.fitko.fitconnect.api.domain.model.submission.SubmissionForPickup;
import de.fitko.fitconnect.api.domain.model.submission.SubmissionSubmit;
import de.fitko.fitconnect.api.domain.validation.ValidationResult;

import java.util.List;
import java.util.UUID;

/**
 * A technical system that accepts submissions on the administration side.
 *
 * @see <a href="https://docs.fitko.de/fit-connect/docs/receiving/overview">Receiving Submissions</a>
 */
public interface Subscriber {

    /**
     * Authenticates the {@link Subscriber} against the FitConnect-Auth-API and retrieves an {@link OAuthToken}.
     *
     * @param clientId     a unique client identifier
     * @param clientSecret the applications secret key
     * @param scope        1 or more client scopes that determine if a submission is accepted by the client
     * @return {@link OAuthToken}, is empty if an error occurred
     */
    OAuthToken retrieveOAuthToken(final String clientId, final String clientSecret, final String... scope);

    /**
     * Decrypts JWE-encrypted string data.
     *
     * @param privateKey       the private key to decrypt the JWE-encrypted string
     * @param encryptedContent the content that is decrypted
     * @return the decrypted content as byte[]
     */
    byte[] decryptStringContent(final RSAKey privateKey, final String encryptedContent);

    /**
     * Polls for available {@link SubmissionSubmit}s on the given destinationId
     *
     * @param destinationId restricts the query to a specific destination
     * @param limit         number of submissions in result (max. is 500)
     * @param offset        position in the dataset
     * @return list of found submissions
     */
    List<SubmissionForPickup> pollAvailableSubmissions(UUID destinationId, int limit, int offset);

    /**
     * Gets a specific {@link SubmissionSubmit}.
     *
     * @param submissionId the unique identifier of a {@link SubmissionSubmit}
     * @return the optional submission, is empty if none was found
     */
    Submission getSubmission(UUID submissionId);

    /**
     * @param submissionId
     * @param announcedAttachments
     * @return
     */
    List<Attachment> fetchAttachments(UUID submissionId, List<UUID> announcedAttachments);

    /**
     * @param caseId
     * @return
     */
    ValidationResult validateEventLog(UUID caseId);

    /**
     * Validates the {@link Metadata} structure against a given JSON-schema to ensure its correctness.
     *
     * @param metadata the {@link Metadata} object that is validated
     * @return a {@link ValidationResult}, contains error if the {@link Metadata} doesn't match the schema
     */
    ValidationResult validateMetadata(Metadata metadata);

    /**
     * @param data
     * @return
     */
    ValidationResult validateData(Data data);

    /**
     * @param attachments
     * @return
     */
    ValidationResult validateAttachments(List<Attachment> attachments);

    /**
     *
     */
    void confirmValidSubmission();
}
